require 'spec_helper'

describe ForteRuby::API::Object do
  subject { ForteRuby::API::Object.new }
  let(:attributes) {{ first_name: 'Bob' }}

  describe '#delete' do
    before { allow(subject).to receive(:url).and_return('delete_url') }

    it 'sends delete request' do
      expect(subject).to receive(:send_request).with('delete_url', :delete)
      subject.delete
    end

    it 'returns response' do
      allow(subject).to receive(:send_request).and_return('success')
      expect(subject.delete).to eq('success')
    end
  end

  describe '#update' do
    before do
      allow(subject).to receive(:set_attributes)
      allow(subject).to receive(:save)
    end

    it 'sets attributes before save' do
      expect(subject).to receive(:set_attributes).with(attributes)
      subject.update(attributes)
    end

    it 'calls save' do
      expect(subject).to receive(:save)
      subject.update(attributes)
    end

    it 'returns response' do
      allow(subject).to receive(:save).and_return('success')
      expect(subject.update(attributes)).to eq('success')
    end
  end

  describe '#save' do
    let(:result) { Resultable::Result.new }

    before do
      allow(subject).to receive(:url).and_return('save_url')
      allow(subject).to receive(:send_request).and_return(result)
      allow(subject).to receive(:api_attributes_hash).and_return(attributes)
    end

    context 'for persisted object' do
      it 'sends put request' do
        expect(subject).to receive(:send_request).with('save_url', :put, attributes).and_return(result)
        subject.save
      end

      it 'returns result' do
        allow(subject).to receive(:send_request).and_return(result)
        expect(subject.save).to eq(result)
      end
    end

    context 'for new object' do
      before { subject.new = true }

      it 'sends post request' do
        expect(subject).to receive(:send_request).with('save_url', :post, attributes).and_return(result)
        subject.save
      end

      it 'returns result' do
        allow(subject).to receive(:send_request).and_return(result)
        expect(subject.save).to eq(result)
      end

      context 'if response is successful' do
        before do
          result.successful!
          result.response = attributes
        end

        it 'sets attributes from resposne' do
          expect(subject).to receive(:set_attributes).with(attributes.merge(new: false))
          subject.save
        end
      end
    end
  end
end
