require 'spec_helper'

describe ForteRuby::Resourses::Customer do
  context 'for existed customer' do
    subject do
      VCR.use_cassette('customers/all/many') do
        ForteRuby::Resourses::Customers.new.all.first
      end
    end

    describe '#update' do
      it 'returns successful result' do
        VCR.use_cassette('customer/update/success') do
          expect(subject.update(first_name: 'Mike').successful?).to eq(true)
        end
      end

      it 'sets first name for customer object' do
        VCR.use_cassette('customer/update/success') do
          subject.update(first_name: 'Mike')
          expect(subject.first_name).to eq('Mike')
        end
      end

      context 'for invalid data' do
        let(:result) { subject.update(status: 'unexisted') }

        it 'returns unsuccessful result' do
          VCR.use_cassette('customer/update/error') do
            expect(result.successful?).to eq(false)
          end
        end

        it 'sets error for result object' do
          VCR.use_cassette('customer/update/error') do
            expect(result.response).to include("Error[2]: Unexpected value for CustomerStatus")
          end
        end
      end
    end

    describe '#delete' do
      it 'returns successful result' do
        VCR.use_cassette('customer/delete/success') do
          expect(subject.delete.successful?).to eq(true)
        end
      end
    end
  end

  context 'create new customer' do
    subject { ForteRuby::Resourses::Customers.new.build_from(first_name: 'Mario') }
    
    it 'returns successful result' do
      VCR.use_cassette('customer/create/success') do
        expect(subject.save.successful?).to eq(true)
      end
    end

    it 'sets first name for customer object' do
      VCR.use_cassette('customer/create/success') do
        subject.save
        expect(subject.first_name).to eq('Mario')
      end
    end

    it 'sets customer token for customer object' do
      VCR.use_cassette('customer/create/success') do
        subject.save
        expect(subject.customer_token).to eq('cst_oygCxFyKj06Tglzm_8JyeQ')
      end
    end

    context 'for invalid data' do
      let(:result) { ForteRuby::Resourses::Customers.new.build_from({}).save }

      it 'returns unsuccessful result' do
        VCR.use_cassette('customer/create/error') do
          expect(result.successful?).to eq(false)
        end
      end

      it 'sets error for result object' do
        VCR.use_cassette('customer/create/error') do
          expect(result.response).to eq("These fields, first_name, last_name, and company_name, cannot all be null. A customer's personal name and/or company name is required.")
        end
      end
    end
  end
end
