require 'spec_helper'

describe ForteRuby::Resourses::Customers do
  subject { ForteRuby::Resourses::Customers.new }

  describe '#all' do
    it 'returns array of customers objects' do
      VCR.use_cassette('customers/all/many') do
        expect(subject.all.first.class).to eq(ForteRuby::Resourses::Customer)
      end
    end

    it 'sets attributes for each customer' do
      VCR.use_cassette('customers/all/many') do
        expect(subject.all.first.first_name).to eq('Bob')
      end
    end

    it 'returns empty array if there are no customers' do
      VCR.use_cassette('customers/all/empty') do
        expect(subject.all).to eq([])
      end
    end
  end

  describe '#find_by_id' do
    it 'returns customer if it exists' do
      VCR.use_cassette('customers/find_by_id/exists') do
        expect(subject.find_by_id('cst_5I5j8RuSCU6kmt3_miCtZQ').first_name).to eq('Mike')
      end
    end

    it 'returns error if it does not exist' do
      VCR.use_cassette('customers/find_by_id/does_not_exist') do
        expect(subject.find_by_id('cst_5I5j8RuSCU6kmt3_miCtZd').response).to eq('This user does not have permission to access one or more of the requested Ids/Tokens.')
      end
    end
  end

  describe '#filter' do
    it 'returns customers matched filter' do
      VCR.use_cassette('customers/filter/exist') do
        expect(subject.filter(first_name: 'Mike').first.first_name).to eq('Mike')
      end
    end

    it 'returns empty array if there are no customers matched filter' do
      VCR.use_cassette('customers/filter/do_not_exist') do
        expect(subject.filter(first_name: 'Mike', last_name: 'Donnals')).to be_empty
      end
    end
  end
end
